/**************************************************************************************************
 * THE OMEGA LIB PROJECT
 *-------------------------------------------------------------------------------------------------
 * Copyright 2010-2015		Electronic Visualization Laboratory, University of Illinois at Chicago
 * Authors:										
 *  Alessandro Febretti		febret@gmail.com
 *-------------------------------------------------------------------------------------------------
 * Copyright (c) 2010-2015, Electronic Visualization Laboratory, University of Illinois at Chicago
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without modification, are permitted 
 * provided that the following conditions are met:
 * 
 * Redistributions of source code must retain the above copyright notice, this list of conditions 
 * and the following disclaimer. Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the documentation and/or other 
 * materials provided with the distribution. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR 
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND 
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE  GOODS OR SERVICES; LOSS OF 
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *************************************************************************************************/
#ifndef __DEFAULT_TWO_HANDS_INTERACTOR_H__
#define __DEFAULT_TWO_HANDS_INTERACTOR_H__

#include "omegaToolkitConfig.h"
#include "omega/Actor.h"
#include "omega/ObserverUpdateServiceExt.h"

namespace omegaToolkit {
	///////////////////////////////////////////////////////////////////////////////////////////////
	class OTK_API DefaultTwoHandsInteractor: public Actor
	{
	public:
		DefaultTwoHandsInteractor(): Actor("DefaultTwoHandsInteractor") {}

		void initialize(const String& observerUpdateServiceName);

		virtual void handleEvent(const Event& evt);

		int getActiveUserId();
		void setActiveUserId(int value);

	private:
		ObserverUpdateServiceExt* myObserverUpdateService;
		int myActiveUserId;
		
		float myInitialScale;
		
		Quaternion myInitialOrientation;
		Vector3f myInitialHandDirection;
		Vector3f myInitialPosition;

		Vector3f myLeftHand;
		Vector3f myRightHand;
		bool myHandsValid;
	};

	///////////////////////////////////////////////////////////////////////////////////////////////
	inline int DefaultTwoHandsInteractor::getActiveUserId()
	{ return myActiveUserId; }

	///////////////////////////////////////////////////////////////////////////////////////////////
	inline void DefaultTwoHandsInteractor::setActiveUserId(int value)
	{ myActiveUserId = value; }
}; // namespace omegaToolkit

#endif